/*******************************************************************************
 * Copyright (c) 2006 Oracle. All rights reserved.
 * This program and the accompanying materials are made available under the terms of
 * the Eclipse Public License v1.0, which accompanies this distribution and is available at
 * http://www.eclipse.org/legal/epl-v10.html.
 ******************************************************************************/
package org.eclipse.dali.example.jsf.inventory.setup;

import java.util.HashMap;
import java.util.Map;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

/**
 * Abstract class for command line JPA programs.  The login
 * information is, by default, picked up from the persistence.xml
 * in the classpath but can be overridden on the command line.
 * 
 * The <code>run</code> template method provides a vehicle for subblasses
 * to add command line arguments and perform additional processing. 
 * 
 * @author shsmith
 *
 */
public abstract class JPACommandLine {

	protected static final String URL = "url";
	protected static final String DRIVER = "driver";
	protected static final String PASSWORD = "password";
	protected static final String USER = "user";
	protected static final String PUNAME = "puname";
	protected static final String DEFAULT_PUNAME = "default";

	protected EntityManagerFactory emf;

	public JPACommandLine() {
		super();
	}

	protected Options getOptions() {
		Option userOption = OptionBuilder.withArgName(USER)
			.hasArg()
			.isRequired(false)
			.withDescription("database user id")
			.create(USER);
		Option passwordOption = OptionBuilder.withArgName(PASSWORD)
			.hasArg()
			.isRequired(false)
			.withDescription("database user password")
			.create(PASSWORD);
		Option driverOption = OptionBuilder.withArgName(DRIVER)
			.hasArg()
			.isRequired(false)
			.withDescription("database driver class")
			.create(DRIVER);
		Option urlOption = OptionBuilder.withArgName(URL)
			.hasArg()
			.isRequired(false)
			.withDescription("database connect string")
			.create(URL);
		Option puName = OptionBuilder.withArgName(PUNAME)
			.hasArg()
			.isRequired(false)
			.withDescription("name of the persistence unit to create")
			.create(PUNAME);
		Options options = new Options();
		options.addOption(userOption);
		options.addOption(passwordOption);
		options.addOption(driverOption);
		options.addOption(urlOption);
		options.addOption(puName);
		return options;
	}

	protected Map<String, String> buildProperties(CommandLine line) {
		Map<String, String> properties = new HashMap<String, String>();
		if (line.hasOption(USER)) {
			properties.put("toplink.jdbc.user", line.getOptionValue(USER));
		}
		if (line.hasOption(PASSWORD)) {
			properties.put("toplink.jdbc.password", line.getOptionValue(PASSWORD));
		}
		if (line.hasOption(DRIVER)) {
			properties.put("toplink.jdbc.driver", line.getOptionValue(DRIVER));
		}
		if (line.hasOption(URL)) {
			properties.put("toplink.jdbc.url", line.getOptionValue(URL));		
		}
		return properties;
	}

	/**
	 * Subclasses should call <code>run</code> from their
	 * main.
	 * 
	 * @param args
	 */
	protected void run(String[] args) {
	    CommandLineParser parser = new GnuParser();
	    CommandLine line = null;
		try {
	        line  = parser.parse(getOptions(), args);
	    }
	    catch( ParseException exp ) {
	        System.err.println( "Parsing failed.  Reason: " + exp.getMessage() );
	        return;
	    }
	
		Map<String, String> properties = buildProperties(line);
		String puName = getPUName(line);
		augmentProperties(properties);
		createEntityManagerFactory(puName, properties);
		perform(line);
		closeEntityManagerFactory();
	}

	private String getPUName(CommandLine line) {
		if (line.hasOption(PUNAME)) {
			return line.getOptionValue(PUNAME);
		} else {
			return DEFAULT_PUNAME;
		}
	}

	protected void createEntityManagerFactory(String puName, Map<String, String> properties) {
		emf = Persistence.createEntityManagerFactory(puName, properties);
	}

	protected void closeEntityManagerFactory() {
		emf.close();
	}
	
	/**
	 * Perform any necessary persistence operations
	 * @param line
	 */
	protected void perform(CommandLine line) {
	}

	/**
	 * Add any properties that are required to create the 
	 * EntityManager factory.
	 * 
	 * @param properties
	 */
	protected void augmentProperties(Map<String, String> properties) {
	}

}